// Generated by CoffeeScript 1.12.4

/*
 * Flare&Para
 *   あかつきみさき(みくちぃP)

 * このスクリプトについて
 *   フレア/パラを追加するスクリプト.
 *   各種パラメーターを指定し,Applyで新規レイヤーとして追加します.
 *   エフェクトには「描画」カテゴリの「グラデーション」を使用しています(CS6以前ではエフェクト名がカラーカーブですが,機能は同一のものです).

 *   エフェクトにグラデーションが適用されている場合,対象のレイヤーを最後に選択した状態で各種パラメーターを操作することができます.

 *   メインパレット(大1つ)
 *     クリック: カラーピッカーによる色の変更
 *   サブパレット(小3つ)
 *     クリック: カラーピッカーによる色の変更
 *     Alt+クリック: メインパレットへ色を移す

 *   ・色
 *     フレア・パラの色を指定します.
 *   ・不透明度
 *     レイヤーの不透明度を指定します.
 *   ・モード
 *     合成モードを指定します.パラの場合は乗算,フレアの場合はスクリーンになります.
 *   ・グラデーションのシェイプ
 *     グラデーションの形状を指定します.
 *   ・グラデーションの拡散
 *     グラデーションの拡散を指定します.
 *   ・光源方向
 *     光源の方向を8方向+中心から指定します.
 *     光源方向が中心の場合は,グラデーションのシェイプが放射状になります.

 * 謝辞
 *   このスクリプトは以下のものをリメイクするという理念の下に作成しています. 基本的な操作方法やエフェクト追加の概念は以下に基づいています.
 *     「アニメ撮影におけるフレア/パラのエフェクトオペレーションを高速化するスクリプトパネル：FP-Launcher」
 *     http://ae-users.com/jp/resources/2010/03/%E3%82%A2%E3%83%8B%E3%83%A1%E6%92%AE%E5%BD%B1%E3%81%AB%E3%81%8A%E3%81%91%E3%82%8B%E3%83%95%E3%83%AC%E3%82%A2%E3%83%91%E3%83%A9%E3%81%AE%E3%82%A8%E3%83%95%E3%82%A7%E3%82%AF%E3%83%88%E3%82%AA%E3%83%9A/

 *   FP-Launcherとの主な違いについて
 *     追加するApplyボタンの追加.
 *     色の取得をRGBスライダーとクリックでのピッカー表示にした.
 *     使用するエフェクトをグラデーションにした.
 *     レイヤーの不透明度に対応.
 *     グラデーションの拡散に対応.
 *     Undoの改善.

 * ライセンスについて
 *   MIT License

 * 使用方法
 *   ScriptUIフォルダーにスクリプトを入れてウィンドウより実行か,スクリプトファイルの実行より実行してください.

 * 動作環境
 *   Adobe After Effects CS6以上

 * バージョン情報
 *   2018/01/25 Ver 1.3.0 Update
 *     グラデーションが適用されていないレイヤーを選択した状態で方向ボタンを押すとエラーが出る問題の修正.
 *     一部UIサイズの調整.
 *     一部アイコンの変更.
 *     色変更時,アラートが出る問題の修正.
 *     対応バージョンの変更.

 *   2016/04/18 Ver 1.2.1 Update
 *     サブパレットの色をメインパレットに移すコマンドをCtrlからAltに変更した.

 *   2016/04/11 Ver 1.2.0 Update
 *     パレットの色を環境設定に保存するようにした.

 *   2016/04/11 Ver 1.1.0 Update
 *     サブパレットを3つ追加した(サブパレットをCtrl+クリックで,メインパレットに色を反映).
 *     色取得時,特定色が正しく反映されない問題の修正.

 *   2014/05/01 Ver 1.0.0 Release
 */


/*Data */

(function() {
  var ADBE_EFFECT_PARADE, ADBE_MARKER, ADBE_OPACITY, ADBE_RAMP, ADBE_TRANSFORM_GROUP, ARROW_ICON, COLOR_MAX_VALUE, END_COLOR_MATCHNAME, END_OF_RAMP_MATCHNAME, ERR_PICKER, FPLData, LINEAR_RAMP, MAIN_PALETTE_COLOR, OPACITY_MATCHNAME, RADIAL_RAMP, RAMP_SCATTER_MATCHNAME, RAMP_SHAPE_MATCHNAME, RGBColor, START_COLOR_MATCHNAME, START_OF_RAMP_MATCHNAME, SUB_PALETTE1_COLOR, SUB_PALETTE2_COLOR, SUB_PALETTE3_COLOR, _STRINGS_EN, _STRINGS_JP, buildUI, colorStringToArray, getLocalizedText, isCompActive, isLayerSelected, runAEVersionCheck, thisColor,
    slice = [].slice;

  FPLData = (function() {
    var canRunVersionName, canRunVersionNumber, scriptName, scriptURLName, scriptURLVersion, scriptVersionNumber;
    scriptName = "Flare&Para";
    scriptURLName = "FlareandPara";
    scriptVersionNumber = "1.3.0";
    scriptURLVersion = 1.30;
    canRunVersionNumber = 11.0;
    canRunVersionName = "CS6";
    return {
      getScriptName: function() {
        return scriptName;
      },
      getScriptURLName: function() {
        return scriptURLName;
      },
      getScriptVersionNumber: function() {
        return scriptVersionNumber;
      },
      getScriptURLVersion: function() {
        return scriptURLVersion;
      },
      getCanRunVersionNumber: function() {
        return canRunVersionNumber;
      },
      getCanRunVersionName: function() {
        return canRunVersionName;
      }
    };
  })();


  /*class */

  RGBColor = (function() {
    var normalized;

    RGBColor.BLACK = [0, 0, 0];

    RGBColor.WHITE = [1, 1, 1];

    RGBColor.RED = [1, 0, 0];

    RGBColor.GREEN = [0, 1, 0];

    RGBColor.BLUE = [0, 0, 1];

    normalized = function(color) {
      var i, j, ref;
      for (i = j = 0, ref = color.length; j < ref; i = j += 1) {
        if (isNaN(color[i])) {
          color[i] = 0.0;
          continue;
        }
        if (color[i] > 1.0) {
          color[i] = 1.0;
        } else if (color[i] < 0.0) {
          color[i] = 0.0;
        }
      }
      return color;
    };

    function RGBColor() {
      var colorValue;
      colorValue = 1 <= arguments.length ? slice.call(arguments, 0) : [];
      if ((colorValue != null) && colorValue instanceof Array) {
        colorValue = normalized(colorValue);
      } else {
        colorValue = [0, 0, 0];
      }
      this.setColorValue = (function(_this) {
        return function(newColor) {
          return colorValue = normalized(newColor);
        };
      })(this);
      this.getColorValue = (function(_this) {
        return function() {
          return colorValue;
        };
      })(this);
    }

    return RGBColor;

  })();


  /*prototype */

  Button.prototype.drawRect = function(color) {
    if (!(color instanceof Array)) {
      return;
    }
    if (color.length < 3) {
      return;
    }
    this.DRAW_MARGINS = 1;
    this.normalized = function(color) {
      var i, j, ref;
      for (i = j = 0, ref = color.length; j < ref; i = j += 1) {
        if (color[i] > 1.0) {
          color[i] = 1.0;
        } else if (color[i] < 0) {
          color[i] = 0.0;
        }
      }
      return color;
    };
    this.reDraw = function() {
      this.enabled = false;
      this.enabled = true;
    };
    this.fillBrush = this.graphics.newBrush(this.graphics.BrushType.SOLID_COLOR, this.normalized(color));
    this.onDraw = function() {
      this.graphics.drawOSControl();
      this.graphics.rectPath(this.DRAW_MARGINS, this.DRAW_MARGINS, this.size[0] - this.DRAW_MARGINS * 2, this.size[1] - this.DRAW_MARGINS * 2);
      this.graphics.fillPath(this.fillBrush);
      if (this.text) {
        this.graphics.drawString(this.text, this.textPen, (this.size[0] - this.graphics.measureString(this.text, this.graphics.font, this.size[0])[0]) / 2, 3, this.graphics.font);
      }
    };
    this.reDraw();
  };

  Button.prototype.drawEllipse = function(color) {
    if (!(color instanceof Array)) {
      return;
    }
    if (color.length < 3) {
      return;
    }
    this.DRAW_MARGINS = 1;
    this.normalized = function(color) {
      var i, j, ref;
      for (i = j = 0, ref = color.length; j < ref; i = j += 1) {
        if (color[i] > 1.0) {
          color[i] = 1.0;
        } else if (color[i] < 0) {
          color[i] = 0.0;
        }
      }
      return color;
    };
    this.reDraw = function() {
      this.enabled = false;
      this.enabled = true;
    };
    this.fillBrush = this.graphics.newBrush(this.graphics.BrushType.SOLID_COLOR, this.normalized(color));
    this.onDraw = function() {
      this.graphics.drawOSControl();
      this.graphics.ellipsePath(this.DRAW_MARGINS, this.DRAW_MARGINS, this.size[0] - this.DRAW_MARGINS * 2, this.size[1] - this.DRAW_MARGINS * 2);
      this.graphics.fillPath(this.fillBrush);
      if (this.text) {
        this.graphics.drawString(this.text, this.textPen, (this.size[0] - this.graphics.measureString(this.text, this.graphics.font, this.size[0])[0]) / 2, 3, this.graphics.font);
      }
    };
    this.reDraw();
  };

  Property.prototype.setValueAtSmart = function(time, newValue) {
    var err;
    try {
      if (this.hasMax || this.hasMin) {
        if (this.maxValue < newValue) {
          newValue = this.maxValue;
        } else if (this.minValue > newValue) {
          newValue = this.minValue;
        }
      }
      if (this.numKeys === 0) {
        this.setValue(newValue);
      } else {
        this.setValueAtTime(time, newValue);
      }
    } catch (error) {
      err = error;
      if (this.numKeys === 0) {
        this.setValue(newValue);
      } else {
        this.setValueAtTime(time, newValue);
      }
    }
  };

  Settings.prototype.initSetting = function(sectionName, keyName, value) {
    if (!app.settings.haveSetting(sectionName, keyName)) {
      app.settings.saveSetting(sectionName, keyName, value);
    }
    return app.settings.getSetting(sectionName, keyName);
  };

  Number.prototype.parseColorArray = function() {
    var b, g, r, temp;
    temp = this;
    r = Math.floor(temp / 65536);
    g = Math.floor((temp / 65536 - r) * 256);
    b = temp - (r * 65536) - (g * 256);
    return [r / 255, g / 255, b / 255];
  };

  Array.prototype.parseColorNumber = function() {
    return Number((this[0] * 255 * Math.pow(256, 2)) + (this[1] * 255 * Math.pow(256, 1)) + (this[2] * 255 * Math.pow(256, 0)));
  };

  colorStringToArray = function(color) {
    var b, g, r;
    r = Math.floor(color / 65536);
    g = Math.floor((color / 65536 - r) * 256);
    b = color - (r * 65536) - (g * 256);
    return [r / 255, g / 255, b / 255];
  };

  MAIN_PALETTE_COLOR = app.settings.initSetting(FPLData.getScriptURLName(), "MAIN_PALETTE_COLOR", [1, 1, 1].parseColorNumber());

  SUB_PALETTE1_COLOR = app.settings.initSetting(FPLData.getScriptURLName(), "SUB_PALETTE1_COLOR", [1, 0, 0].parseColorNumber());

  SUB_PALETTE2_COLOR = app.settings.initSetting(FPLData.getScriptURLName(), "SUB_PALETTE2_COLOR", [0, 1, 0].parseColorNumber());

  SUB_PALETTE3_COLOR = app.settings.initSetting(FPLData.getScriptURLName(), "SUB_PALETTE3_COLOR", [0, 0, 1].parseColorNumber());


  /*resource */

  COLOR_MAX_VALUE = 255;

  ADBE_TRANSFORM_GROUP = "ADBE Transform Group";

  ADBE_OPACITY = "ADBE Opacity";

  ADBE_EFFECT_PARADE = "ADBE Effect Parade";

  ADBE_RAMP = "ADBE Ramp";

  ADBE_MARKER = "ADBE Marker";

  OPACITY_MATCHNAME = ADBE_OPACITY;

  START_OF_RAMP_MATCHNAME = "ADBE Ramp-0001";

  END_OF_RAMP_MATCHNAME = "ADBE Ramp-0003";

  START_COLOR_MATCHNAME = "ADBE Ramp-0002";

  END_COLOR_MATCHNAME = "ADBE Ramp-0004";

  RAMP_SHAPE_MATCHNAME = "ADBE Ramp-0005";

  RAMP_SCATTER_MATCHNAME = "ADBE Ramp-0006";

  ERR_PICKER = "-1";

  LINEAR_RAMP = 1;

  RADIAL_RAMP = 2;

  _STRINGS_JP = {
    AUTHOR: "あかつきみさき (SUNRISE MOON)",
    HELP: "フレア/パラを追加するスクリプト.\n謝辞: FP-Launcher",
    FLARE_C: "フレア",
    FLARE_M: "*---フレア---",
    PARA_C: "パラ",
    PARA_M: "*---パラ---"
  };

  _STRINGS_EN = {
    AUTHOR: "Misaki_Akatsuki (SUNRISE MOON)",
    HELP: "Add Flare/Para script.\nSpecial thanks: FP-Launcher",
    FLARE_C: "Flare",
    FLARE_M: "*---Flare---",
    PARA_C: "Para",
    PARA_M: "*---Para---"
  };

  ARROW_ICON = ["\u0089PNG\r\n\x1A\n\x00\x00\x00\rIHDR\x00\x00\x00\x12\x00\x00\x00\x12\b\x06\x00\x00\x00V\u00CE\u008EW\x00\x00\x00\tpHYs\x00\x00\x00\x01\x00\x00\x00\x01\x018\"\u00F4@\x00\x00\x00$zTXtCreator\x00\x00\b\u0099sL\u00C9OJUpL+I-RpMKKM.)\x06\x00Az\x06\u00CEjz\x15\u00C5\x00\x00\x01\tIDAT8\u008D\u00E5\u0093!\u008E\u00C2@\x18F\x1F\u00A4\u00A6\x06Q\u008D\u00AA\u00A8\u00F8I@L\u0093\u009A\u009A\u00DE\x00\u00D1\x0BT\u008C\u00E7\x06Hd\u00A9\u00E6\x02\u009C\u0082;`*\u00ABIEE+F\u00D0dV-a\u00D3\u00C9\x06\u0082\u00DA\u00ECs\u00F3'\u00DF\u009Bof2\u00F0\x7F\u00D8l66\fC\u00FB\u00B1\u00E4|>\u00DB\u00C3\u00E1\u00F0\u00B6l\u00FE\u00BC\x18\u00C7\x11c\f\"\u0082\u00D6\u009Awd3W\u00AB\u00DDnG\x10\x04\u00D4u\u00CD\u00E9t\x02@kM\u00DB\u00B6\u0094e9\u00C98E.\x19\u0080\u0088\u00D0\u00F7=\u00FB\u00FD\u009E\u00A6i&\u00B9\u00F9\u00C4\x02\\\u00AF\u00D7\u00D9\u00F1x\u00A4\u00EB:D\x04\x11\x01`\u00B1X\u00B0^\u00AF\u009DGs\u008A\x00\u0086a\u00E0v\u00BBM\u00E6I\u0092\u00BC.\n\u00C3\u00D0j\u00AD\x1FM\u009E\x11\x11V\u00AB\u00D5\u00E4\x11\u009C\"\u00CF\u00F30\u00C6p\u00BF\u00DF\u009D\u00BB\u00BBZ9/\u00FB\u009B(\u008A\u00ACR\u008A4MY.\u0097\u008Fy\u00D7u\x14E\u00F1#\u00FB\u00AB\u00E8\u00998\u008Em\u0096e(\u00A5\u00F0}\u009F\u00AA\u00AA\u00B8\\./\u00E7\u009Dl\u00B7[\u009B\u00E7\u00F9g\u00DF\u00E8\u00EF\u00F0\x05/-Z\u00BC\b\u00BC\u008B\u00DB\x00\x00\x00\x00IEND\u00AEB`\u0082", "\u0089PNG\r\n\x1A\n\x00\x00\x00\rIHDR\x00\x00\x00\x12\x00\x00\x00\x12\b\x06\x00\x00\x00V\u00CE\u008EW\x00\x00\x00\tpHYs\x00\x00\x00\x01\x00\x00\x00\x01\x018\"\u00F4@\x00\x00\x00$zTXtCreator\x00\x00\b\u0099sL\u00C9OJUpL+I-RpMKKM.)\x06\x00Az\x06\u00CEjz\x15\u00C5\x00\x00\x00\u00BBIDAT8\u008D\u00E5\u0093!\x0E\u00840\x10E_7{\u0082j\u00EE\u00C1\u00B5\u00D0\u00DC\u00A3G\x01\x01\x0ERA\u0082\u00A1I\u0083\x03\u00CD\t\x10\x15]\u00BB\t\x03E`6;\u00F2\u00E7\u00CF\u009B?\u0093\f\u00FC\\\x19c\u00A21&\u00A6|\u00EF\u0094!\u00CB\u00B2[\x03_\u00B7\\\u00FF\rR\u0092X\u0096e\u00CC\u00F3\\lX\u00D7\u0095\u00A2(\x0E}b\u00A2i\u009AN';\u00E7D]\x04UU\u00A5\u00B6m;\u00E8!\x04\u0086a\u00B8\x0F\x02\u00E8\u00FB\u00FE\u00A0Yk\u00F1\u00DE\u008B\u00E78\x05Yk\u00D9\u00F7=\tO\u0082\u0096eQ\u00DFk8\u00E7\x18\u00C7QLs\t\x02h\u009A\u0086\x10\x02\x00m\u00DB^Y\u00AF\x7F\u00CD{\u00AF\u00E6y\u008EZk\u00BA\u00AE;M\u0093\x04\x01\u00D4u\u008D\u00D6:e{\u00AE>c\u00CFF\u0091\u00D5\u009A>\u00B1\x00\x00\x00\x00IEND\u00AEB`\u0082", "\u0089PNG\r\n\x1A\n\x00\x00\x00\rIHDR\x00\x00\x00\x12\x00\x00\x00\x12\b\x06\x00\x00\x00V\u00CE\u008EW\x00\x00\x00\tpHYs\x00\x00\x00\x01\x00\x00\x00\x01\x018\"\u00F4@\x00\x00\x00$zTXtCreator\x00\x00\b\u0099sL\u00C9OJUpL+I-RpMKKM.)\x06\x00Az\x06\u00CEjz\x15\u00C5\x00\x00\x00\u00FAIDAT8\u008D\u00ED\u0091!o\u0084@\x14\u0084\u00A7\u00CDi,f-$\x04\u00BB\u00C9r\x06\u0081\u00C5\u00F2?\u00F0\u00FC\x06\f\u0082\u00FF\u0080\u00C0 \u00D1\b\u00D4\u0092`\u00D1 A @@\u00F2\u00AA.9\n\u00DB^S\u00D9\u008E|o\u00E7{\u0099Y\u00E0_\u00CF\u00E2\u009C\u00D3\u00AF!Q\x14Q\u009E\u00E7\u00E4\u00BA\u00EE\x01\u00F6\u00FES\u0088\x10\x02\u00FB\u00BEcY\u0096\u00D7LA\x10P\u009A\u00A6d\u009A&= EQP\u0096et\u00BF\u00DFO\u00D1\u00DET\u00A08\u008E\u00C90\f\u00F4}\u008Fa\x18 \u0084\u00C0\u00BA\u00AEH\u0092\x04u]\u009F|\u00B7+\b\u00E7\u009C\f\u00C3\x00\x000\u00C6\u00C0\x18\u00FB\x12\x02(:r\x1C\u00E74\x1B\u00C7\x11\u00E38\u00AA\x02\\\u0083\u0084\x10\u00A7\x19c\fa\x18\u00E2\u00D1\u00D9\u00B7 \u00DF\u00F7I\u00D3\u00B4\u00CB\u00AB\u00F3<C\u00B5;u\u00F49\u00D64M\u00A8\u00AA\nRJ\u00B4m\u00AB\u00FC\u009C\x03\u00C8\u00B6m\u00B2,\x0B\u00DB\u00B6\u00A1i\x1AH)Q\u0096\u00A5\u00D2\u00FC\u00AC\u00C3#\u00CF\u00F3H\u00D7uH)\u00D1u\u00DDK\u0080?\u00A0\x0F\u008A\u0092h#\u00C4ogv\x00\x00\x00\x00IEND\u00AEB`\u0082", "\u0089PNG\r\n\x1A\n\x00\x00\x00\rIHDR\x00\x00\x00\x12\x00\x00\x00\x12\b\x06\x00\x00\x00V\u00CE\u008EW\x00\x00\x00\tpHYs\x00\x00\x00\x01\x00\x00\x00\x01\x018\"\u00F4@\x00\x00\x00$zTXtCreator\x00\x00\b\u0099sL\u00C9OJUpL+I-RpMKKM.)\x06\x00Az\x06\u00CEjz\x15\u00C5\x00\x00\x00\u00ABIDAT8\u008D\u00ED\u008F-\x0E\u00840\x14\u0084?6k\u00D0X,\u0082Kp'\x0E\u00C3=@RQj_\x02\x02\x01\x02\u00C3\rj\u0091]E\x13\u00F6'e\u00CDf\x05\u009Fz\u00C9\u009B\u0099\u00CC\u00C0\u00C5\u00FFP\u0096\u00A5+\u008A\u00C2\u0085tQH\u00D04\u008D\x03X\u00D7\x15c\fu]\u00BF\u00F5\u009C\x0E\u00DA\u00B1\u00D6\u00A2\u00B5FDX\u0096\u00C5\u00FB\u00FDQU\u0095K\u00D34\u0094\u00EB\u00D9\u00B6\r\x11Ak\u00CD8\u008E\u00D1m\x7F$Ir:\x04 \u008Ec\u00F2<'\u00CB\u00B2c\u00A3O<O\x03\u0098\u00E7\x19c\fm\u00DBz\u00FF\u00FD\u009B\x16\"\u0082R\u008A\u00BE\u00EF_\n\x04\u0083\u00AC\u00B5\f\u00C3@\u00D7uL\u00D3\x14\\p\u00F1\x03\x1E\u0080;D4gS\u00E2\u0094\x00\x00\x00\x00IEND\u00AEB`\u0082", "\u0089PNG\r\n\x1A\n\x00\x00\x00\rIHDR\x00\x00\x00\x12\x00\x00\x00\x12\b\x06\x00\x00\x00V\u00CE\u008EW\x00\x00\x00\tpHYs\x00\x00\x00\x01\x00\x00\x00\x01\x018\"\u00F4@\x00\x00\x00$zTXtCreator\x00\x00\b\u0099sL\u00C9OJUpL+I-RpMKKM.)\x06\x00Az\x06\u00CEjz\x15\u00C5\x00\x00\x00\u00ACIDAT8\u008D\u00ED\u00911\n\u0083@\x10E\u009F!\u008D\u008D\u008D\u0085\u00FD\u0082\u0082\b.\u00AC\u0085\u00CD\"\u00BA\u0085\u00AD'\u00F0\x04V^\u00C7\u00F3y\u008F\u00B1\n\x18\x12\u00E3\u00A6\t)|\u00DD0\u00FF\x7F>3p\u00F1?h\u00ADe\u009Eg9\u00D3\x05G\x0B\u00E7\u009Ct]GY\u0096\x00\f\u00C3p\u00A8\x05\u00B8\u00EF\x07\u00A5\u0094\x18c\u00E8\u00FB\u009E$I<;\u00EF\x1A\x15E!\u00D6Z\u00AC\u00B5DQ\u00E4m^\u00D7\u0095i\u009A\x02\u0080\x1B@\u009A\u00A6\u00E4y\u00FEU\b@\x1C\u00C7\u00CF\u008D\x1E4M#m\u00DBRU\u00D5\u008B\u00E9\u00ECFo\u0097Zkq\u00CEQ\u00D75a\x18z\x05}$\u00CB2\x19\u00C7Q\u0096e9}\u00FF\u00C5\x0F\u00D9\x00\u00DCp!\u009CiG^\u00DB\x00\x00\x00\x00IEND\u00AEB`\u0082", "\u0089PNG\r\n\x1A\n\x00\x00\x00\rIHDR\x00\x00\x00\x12\x00\x00\x00\x12\b\x06\x00\x00\x00V\u00CE\u008EW\x00\x00\x00\tpHYs\x00\x00\x00\x01\x00\x00\x00\x01\x018\"\u00F4@\x00\x00\x00$zTXtCreator\x00\x00\b\u0099sL\u00C9OJUpL+I-RpMKKM.)\x06\x00Az\x06\u00CEjz\x15\u00C5\x00\x00\x00\u00FEIDAT8\u008D\u00ED\u0092!n\u0085@\x10\u0086\x7F\x1AN\u00C0\x11\x10\b\x04\u009B\u00B0$\x04\x01\x02\u0083\u00C4p\x10\u00F4\u009E\x02n\u00B1\u0096+\u0080B@\u0082\u00C0 \u00C0`0HV \u00B6\u00AA\u00E4\u00F5\u00C1+i\u00EB\u009A~\u00F2\u009F\u00D9/3\u0093\x05\u00FEy\u00C40\f\x19\u0086\u00A1|\u00CC\u00D4\u00EF\b\u00A2(\u0092\u0094R\u00D8\u00B6\r\u00CE\u00F9\u00A7\u00DA\u00AD\u0088\x10\")\u00A5\u00F0}\x1F\u009A\u00A6\x01\x00\u0084\x10h\u00DB\u00F6^\u00A4\u00EB\u00BA\u00B4,\x0B\u00AE\u00EB\u00C24\u00CDS\u00BDi\x1A\u008C\u00E3\u00A8\u00DC\u008ATUE\x1C\u00C7\u00C7\x04\u00CF\u00D4u}\u00CA\u00DE\u00AE\x1A\u0087aP\u00B2,\u00C3\u00BA\u00AE\u00A7\u00DA\u00B2,(\u00CBRy\u00CE/E\x00\u00D0u\u009D2M\u00D3)\u00AF\u00AA\u00EA\u00B2\u00FF\u00A5\u00881&)\u00A5\x10B\x1C\u0093\u00ED\u00FB~:\u00F2\x07\u00977b\u008CI\u00D7u!\u0084@\u009E\u00E7\u00D8\u00B6\ri\u009Ab\u009Eg\u00F4}\x7FZ\u00EB\u00E5$EQH\u00CE\u00B9\u00F4<\u00EF\u00F8t\u0084\x10\u00E98\u008E\u00FC\u00EA\u00EDA\x10\x04\u0097\u0092\x1F\u0091$\u00C9\u00EF%\x7F\u008Bw\x1C\x0Bi \u00E7\u00A7\u0096\u00BE\x00\x00\x00\x00IEND\u00AEB`\u0082", "\u0089PNG\r\n\x1A\n\x00\x00\x00\rIHDR\x00\x00\x00\x12\x00\x00\x00\x12\b\x06\x00\x00\x00V\u00CE\u008EW\x00\x00\x00\tpHYs\x00\x00\x00\x01\x00\x00\x00\x01\x018\"\u00F4@\x00\x00\x00$zTXtCreator\x00\x00\b\u0099sL\u00C9OJUpL+I-RpMKKM.)\x06\x00Az\x06\u00CEjz\x15\u00C5\x00\x00\x00\u00AAIDAT8\u008D\u00DD\u0092\u00B1\n\u00C3 \x14E/\u00A5\x0E.Y\u00DD\x1D\x02FHF\u0091\u00E8\"\u00C4\u00C1|\u00AD_\u00E7h\u00A7\x16J\u008C&m\u00C9\u00D0\u00BB\u00F9\u00BC\u009Cw\u0086\x07\\\x15)e\u00D6Z\u00E7V\u00EF\u00D6*x\u00EF\x11Bh.\u00AC\u00828\u00E7Y)\u0085q\x1C1MS\u00D5\u00AA\n\u009A\u00E7\x19\u0094R\x00\u0080s\u00EEs#k\u00ED\x1B\u00B4\u00EF\u00FB]\u00AB]\u00D0\u00BA\u00AE\u00991\u00F6z\x13B\u00A0\u0094:od\u008C\u00D9\u00CC\u00BC\u00F7\u00E0\u009C\x17\u00AD\u008A \u00ADu\x1E\u0086a3\u00EF\u00BA\x0EB\u0088\u00E3F%\u009Bg\u0096e\u00D9\u00FD\u00AB&\u00C6\u0098c\u008C\u00DF\x1F\u00E4\u00D1\u00FC1\u00E8\u00DE*\u00A4\u0094~\u00B5\u00EB\u00E2<\x00pb \u009F\u00AF\u00D0S\u00AF\x00\x00\x00\x00IEND\u00AEB`\u0082", "\u0089PNG\r\n\x1A\n\x00\x00\x00\rIHDR\x00\x00\x00\x12\x00\x00\x00\x12\b\x06\x00\x00\x00V\u00CE\u008EW\x00\x00\x00\tpHYs\x00\x00\x00\x01\x00\x00\x00\x01\x018\"\u00F4@\x00\x00\x00$zTXtCreator\x00\x00\b\u0099sL\u00C9OJUpL+I-RpMKKM.)\x06\x00Az\x06\u00CEjz\x15\u00C5\x00\x00\x00\u00FEIDAT8\u008D\u00E5\u00D2!\u008E\u00840\x14\x06\u00E0\u009F\u00CD`9A5\x06\u0081\u00C1p\n\u0090 8\x03'\u00E0\x0E \u00AA\u00B9\x02H\x14\x06A\x13j\x10\u0088jl+0\bDW-\u0099L\u0099\u00CD\u00EC\u00AC\u00DA\u00EC\u00EF\u00DA\u00BE\u00F7\u00E5\u00BD\u00A4\u00C0\u00FFH\x10\x04\u00DA\u00F3<\u00FDN\u00EF\u00ED\u00FE`\u00DB6\u008A\u00A2\u00C0\u00B2,\u009A1\u0086\u00A6i\u00ACW!\u00A3\u0090R\u00AA\t!\x00\u0080m\u00DB0\u00CF3\u00FA\u00BE\u00C74M\u00DF\u00A2\u00C6c\u009A\u00A6:I\x12\u00A3p]W\f\u00C3\x00\u00CE9\u0084\x10F\u00DF\u00C7\u00E3\x05\u00E7\x1C\u00C7q\x18\x10!\x04Q\x14\u00C1q\u009C\u00CB\u0089\fH\ba\u00CD\u00F3l\x14\u00EE\u00FB\u008E\u00AA\u00AA\u009E\u00AEh@\x00\u00C0\x183\u00EE\u00A4\u0094\u0090R^N\u00F3\x14\u00EA\u00BA\u00CERJ\u009D\u0093(\u00A5@\bA\u009E\u00E7p]\u00F7\u00F2{\\B\x000\u008E\u00E3\u00B9NY\u0096/a\u0097q]W\u0087ax6\u00F8\u00BE\u00AF\u00EB\u00BA\u00D6m\u00DBjJ\u00A9\u00FE\x11\u00F6\u0098{,\u008E\u00E3\u00F7\u00A1/,\u00CB\u00B2\u00DF!\x7F+\u009F\x16jyO\u0083\u0093\x18\u008F\x00\x00\x00\x00IEND\u00AEB`\u0082"];

  thisColor = new RGBColor([1, 1, 1]);

  thisColor.setColorValue([1, 1, 1]);


  /*function */

  runAEVersionCheck = function(data) {
    if (parseFloat(app.version) < data.getCanRunVersionNumber()) {
      alert("This script requires After Effects " + (data.getCanRunVersionName()) + " or greater.", data.getScriptName());
      return false;
    } else {
      return true;
    }
  };

  isCompActive = function(selComp) {
    if (!(selComp && selComp instanceof CompItem)) {
      return false;
    } else {
      return true;
    }
  };

  isLayerSelected = function(selLayers) {
    if (selLayers.length === 0) {
      return false;
    } else {
      return true;
    }
  };

  getLocalizedText = function(str) {
    if (app.language === Language.JAPANESE) {
      return str.jp;
    } else {
      return str.en;
    }
  };


  /*ui */

  buildUI = function(thisObj) {
    var CC2014_UI_MARGINS, MAX_ICON_HEIGHT_SIZE, MAX_UI_HEIGHT_SIZE, UIResource, entryFunc, err, reDrawColorPanel, thisUI, undoEntryFunc, updatePanelColor, updateRGBColorPanel, updateRGBColorSlider, updateRGBColorText;
    MAX_UI_HEIGHT_SIZE = 20;
    MAX_ICON_HEIGHT_SIZE = 24;
    CC2014_UI_MARGINS = (parseFloat(app.version) >= 13.1 ? 1 : 0);
    thisUI = (thisObj instanceof Panel ? thisObj : new Window("palette", FPLData.getScriptName(), undefined, {
      resizeable: true
    }));
    UIResource = "group{\n  orientation:\"column\",\n  alignment:[\"fill\",\"top\"],\n  spacing:0,\n  margins:[0,0,0,0],\n\n  colorGrp: Panel{\n    text: \"" + (String(getLocalizedText({
      jp: "色",
      en: "Color"
    }))) + "\",\n    spacing: 0,\n    margins:[2,4,2,2],\n    alignment:[\"fill\",\"top\"],\n    orientation: \"column\",\n\n    paletteGrp: Group{\n      spacing: 0,\n      margins: [0,0,0," + (CC2014_UI_MARGINS * 2) + "],\n      orientation:\"row\",\n      alignment:[\"fill\",\"top\"],\n\n      colorGrp: Group{\n        spacing: 0,\n        margins: [0,2,0,0],\n        orientation:\"column\",\n        alignment:[\"left\",\"top\"],\n\n        setColorBtn: Button{\n          text:\"\",\n          minimumSize:[" + (MAX_UI_HEIGHT_SIZE * 3) + ", " + (MAX_UI_HEIGHT_SIZE * 3) + "],\n          maximumSize:[" + (MAX_UI_HEIGHT_SIZE * 3) + ", " + (MAX_UI_HEIGHT_SIZE * 3) + "],\n          alignment:[\"fill\",\"fill\"],\n          helpTip:\"" + (String(getLocalizedText({
      jp: "メインパレット",
      en: "palette"
    }))) + "\"\n        },\n      },\n\n      controllGrp: Group{\n        spacing: 0,\n        margins: [0,0,0,0],\n        orientation:\"column\",\n        alignment:[\"fill\",\"top\"],\n\n        rGrp: Group{\n          spacing: 0,\n          margins: [0,1,0,0],\n          orientation:\"row\",\n          alignment:[\"fill\",\"top\"],\n\n          colorPanel: Button{\n            text:\"\",\n            minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"left\",\"fill\"],\n            helpTip:\"" + (String(getLocalizedText({
      jp: "赤",
      en: "Red"
    }))) + "\"\n          },\n\n          r: StaticText{\n            text:\"\",\n            minimumSize:[3, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[3, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"left\",\"fill\"],\n            helpTip:\"" + (String(getLocalizedText({
      jp: "赤",
      en: "Red"
    }))) + "\"\n          },\n\n          colorStaticText: StaticText{\n            text:\"R:\",\n            minimumSize:[15, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[15, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"left\",\"fill\"],\n          },\n\n          colorSlider: Slider{\n            minvalue:0,\n            maxvalue:" + COLOR_MAX_VALUE + ",\n            value:" + COLOR_MAX_VALUE + ",\n            minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"fill\",\"fill\"],\n            helpTip:\"\",\n          },\n\n          colorEdit: EditText{\n            text: " + COLOR_MAX_VALUE + ",\n            minimumSize:[40, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[40, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"right\",\"fill\"],\n            helpTip:\"" + (String(getLocalizedText({
      jp: "赤",
      en: "Red"
    }))) + "\"\n          },\n        },\n\n        gGrp: Group{\n          spacing: 0,\n          margins: [0,1,0,0],\n          orientation:\"row\",\n          alignment:[\"fill\",\"top\"],\n\n          colorPanel: Button{\n            text:\"\",\n            minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"left\",\"fill\"],\n            helpTip:\"" + (String(getLocalizedText({
      jp: "緑",
      en: "Green"
    }))) + "\"\n          },\n\n          g: StaticText{\n            text:\"\",\n            minimumSize:[3, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[3, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"left\",\"fill\"],\n            helpTip:\"" + (String(getLocalizedText({
      jp: "緑",
      en: "Green"
    }))) + "\"\n          },\n\n          colorStaticText: StaticText{\n            text:\"G:\",\n            minimumSize:[15, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[15, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"left\",\"fill\"],\n          },\n\n          colorSlider: Slider{\n            minvalue:0,\n            maxvalue:" + COLOR_MAX_VALUE + ",\n            value:" + COLOR_MAX_VALUE + ",\n            minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"fill\",\"fill\"],\n            helpTip:\"\",\n          },\n\n          colorEdit: EditText{\n            text: " + COLOR_MAX_VALUE + ",\n            minimumSize:[40, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[40, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"right\",\"fill\"],\n            helpTip:\"" + (String(getLocalizedText({
      jp: "緑",
      en: "Green"
    }))) + "\"\n          },\n        },\n\n        bGrp: Group{\n          spacing: 0,\n          margins: [0,1,0,0],\n          orientation:\"row\",\n          alignment:[\"fill\",\"top\"],\n\n          colorPanel: Button{\n            text:\"\",\n            minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"left\",\"fill\"],\n            helpTip:\"" + (String(getLocalizedText({
      jp: "青",
      en: "Blue"
    }))) + "\"\n          },\n\n          b: StaticText{\n            text:\"\",\n            minimumSize:[3, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[3, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"left\",\"fill\"],\n            helpTip:\"" + (String(getLocalizedText({
      jp: "青",
      en: "Blue"
    }))) + "\"\n          },\n\n          colorStaticText: StaticText{\n            text:\"B:\",\n            minimumSize:[15, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[15, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"left\",\"fill\"],\n          },\n\n          colorSlider: Slider{\n            minvalue:0,\n            maxvalue:" + COLOR_MAX_VALUE + ",\n            value:" + COLOR_MAX_VALUE + ",\n            minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"fill\",\"fill\"],\n            helpTip:\"\",\n          },\n\n          colorEdit: EditText{\n            text: " + COLOR_MAX_VALUE + ",\n            minimumSize:[40, " + MAX_UI_HEIGHT_SIZE + "],\n            maximumSize:[40, " + MAX_UI_HEIGHT_SIZE + "],\n            alignment:[\"right\",\"fill\"],\n            helpTip:\"" + (String(getLocalizedText({
      jp: "青",
      en: "Blue"
    }))) + "\"\n          },\n        },\n      },\n    },\n\n    a: Panel{\n      spacing: " + CC2014_UI_MARGINS + ",\n      alignment:[\"fill\",\"top\"],\n      minimumSize:[1, 0],\n      maximumSize:[50000, 0],\n    },\n\n    colorCodeGrp: Group{\n      spacing: 0,\n      margins: [0," + (CC2014_UI_MARGINS * 4) + ",0,0],\n      orientation:\"row\",\n      alignment:[\"fill\",\"top\"],\n\n      subColorBtn1: Button{\n        spacing: 0,\n        margins: [0,0,0,0],\n        text:\"\",\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n        maximumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n        alignment:[\"left\",\"fill\"],\n        helpTip:\"" + (String(getLocalizedText({
      jp: "サブパレット1",
      en: "Sub palette 1"
    }))) + "\"\n      },\n\n      a: StaticText{\n        text:\"\",\n        minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_UI_HEIGHT_SIZE + "],\n        maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_UI_HEIGHT_SIZE + "],\n        alignment:[\"left\",\"fill\"],\n        helpTip:\"\"\n      },\n\n      subColorBtn2: Button{\n        spacing: 0,\n        margins: [0,0,0,0],\n        text:\"\",\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n        maximumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n        alignment:[\"left\",\"fill\"],\n        helpTip:\"" + (String(getLocalizedText({
      jp: "サブパレット2",
      en: "Sub palette 2"
    }))) + "\"\n      },\n\n      a: StaticText{\n        text:\"\",\n        minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_UI_HEIGHT_SIZE + "],\n        maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_UI_HEIGHT_SIZE + "],\n        alignment:[\"left\",\"fill\"],\n        helpTip:\"\"\n      },\n\n      subColorBtn3: Button{\n        spacing: 0,\n        margins: [0,0,0,0],\n        text:\"\",\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n        maximumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_UI_HEIGHT_SIZE + "],\n        alignment:[\"left\",\"fill\"],\n        helpTip:\"" + (String(getLocalizedText({
      jp: "サブパレット3",
      en: "Sub palette 3"
    }))) + "\"\n      },\n\n      a: StaticText{\n        text:\"\",\n        minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_UI_HEIGHT_SIZE + "],\n        maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_UI_HEIGHT_SIZE + "],\n        alignment:[\"left\",\"fill\"],\n        helpTip:\"\"\n      },\n\n      colorCodeShape: StaticText{\n        text:\"" + (String(getLocalizedText({
      jp: "カラーコード",
      en: "Color Code"
    }))) + "\",\n        minimumSize:[78, " + MAX_UI_HEIGHT_SIZE + "],\n        maximumSize:[78, " + MAX_UI_HEIGHT_SIZE + "],\n        alignment:[\"left\",\"fill\"],\n        helpTip:\"" + (String(getLocalizedText({
      jp: "カラーコード",
      en: "Color Code"
    }))) + "\"\n      },\n\n      colorCodeShape: StaticText{\n        text:\"" + (String(getLocalizedText({
      jp: "#",
      en: "#"
    }))) + "\",\n        minimumSize:[10, " + MAX_UI_HEIGHT_SIZE + "],\n        maximumSize:[10, " + MAX_UI_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        helpTip:\"" + (String(getLocalizedText({
      jp: "",
      en: ""
    }))) + "\"\n      },\n\n      colorCodeText: EditText{\n        text:\"FFFFFF\",\n        minimumSize:[60, " + MAX_UI_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        helpTip:\"" + (String(getLocalizedText({
      jp: "カラーコード",
      en: "Color Code"
    }))) + "\"\n      },\n    },\n  },\n\n  opacityGrp: Panel{\n    text: \"" + (String(getLocalizedText({
      jp: "不透明度",
      en: "Opacity"
    }))) + "\",\n    spacing: 0,\n    margins:[2,4,2,0],\n    alignment:[\"fill\",\"top\"],\n    orientation: \"row\",\n\n    opacitySlider: Slider{\n      minvalue:0,\n      maxvalue:100,\n      value:100,\n      minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"fill\",\"fill\"],\n      helpTip:\"100\",\n    },\n\n    opacityEdit: EditText{\n      text:\"100\",\n      minimumSize:[60, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[60, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"right\",\"fill\"],\n      helpTip:\"" + (String(getLocalizedText({
      jp: "不透明度",
      en: "Opacity"
    }))) + "\"\n    },\n  },\n\n  blendModeGrp: Panel{\n    text: \"" + (String(getLocalizedText({
      jp: "合成モード",
      en: "Blending Mode"
    }))) + "\",\n    spacing: 0,\n    margins:[2,4,2,0],\n    alignment:[\"fill\",\"top\"],\n    orientation: \"row\",\n\n    paraBtn: RadioButton{\n      text:\"" + (String(getLocalizedText({
      jp: _STRINGS_JP.PARA_C,
      en: _STRINGS_EN.PARA_C
    }))) + "\",\n      value:true,\n      minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"fill\",\"fill\"],\n      helpTip:\"" + (String(getLocalizedText({
      jp: "パラ(乗算)",
      en: "Para(Multiply)"
    }))) + "\",\n    },\n\n    flareBtn: RadioButton{\n      text:\"" + (String(getLocalizedText({
      jp: _STRINGS_JP.FLARE_C,
      en: _STRINGS_EN.FLARE_C
    }))) + "\",\n      value:false,\n      minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"fill\",\"fill\"],\n      helpTip:\"" + (String(getLocalizedText({
      jp: "フレア(スクリーン)",
      en: "Flare(Screen)"
    }))) + "\",\n    },\n  },\n\n  curveTypeGrp: Panel{\n    text: \"" + (String(getLocalizedText({
      jp: "グラデーションのシェイプ",
      en: "Ramp Shape"
    }))) + "\",\n    spacing: 0,\n    margins:[2,4,2,0],\n    alignment:[\"fill\",\"top\"],\n    orientation: \"row\",\n\n    linearBtn: RadioButton{\n      text:\"" + (String(getLocalizedText({
      jp: "直線状",
      en: "Linear Ramp"
    }))) + "\",\n      value:true,\n      minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"fill\",\"fill\"],\n      helpTip:\"\"\n    },\n\n    radiusBtn: RadioButton{\n      text:\"" + (String(getLocalizedText({
      jp: "放射状",
      en: "Radial Ramp"
    }))) + "\",\n      value:false,\n      minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"fill\",\"fill\"],\n      helpTip:\"\"\n    },\n  },\n\n  scatterGrp: Panel{\n    text: \"" + (String(getLocalizedText({
      jp: "グラデーションの拡散",
      en: "Ramp Scatter"
    }))) + "\",\n    spacing: 0,\n    margins:[2,4,2,0],\n    alignment:[\"fill\",\"top\"],\n    orientation: \"row\",\n\n    scatterSlider: Slider{\n      minvalue:0,\n      maxvalue:50,\n      value:0,\n      minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"fill\",\"fill\"],\n      helpTip:\"\",\n    },\n\n    scatterEdit: EditText{\n      text:\"0\",\n      minimumSize:[60, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[60, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"right\",\"fill\"],\n      helpTip:\"Ramp Scatter\"\n    },\n  },\n\n  arrowBtnGrp: Panel{\n    text: \"" + (String(getLocalizedText({
      jp: "光源方向",
      en: "Direction"
    }))) + "\",\n    spacing: 0,\n    margins:[0,4,0,2],\n    alignment:[\"fill\",\"top\"],\n    orientation:\"column\",\n\n    topArrowBtnGrp: Group{\n      spacing: 0,\n      margins: [0,1,0,0],\n      orientation:\"row\",\n      alignment:[\"fill\",\"top\"],\n\n      leftBtn: IconButton{\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        properties:{\n          toggle:true,\n        },\n        helpTip:\"\"\n      },\n\n      a: StaticText{\n        text:\"\",\n        minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        helpTip:\"\"\n      },\n\n      centerBtn: IconButton{\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        properties:{\n          toggle:true,\n        },\n        helpTip:\"\"\n      },\n\n      a: StaticText{\n        text:\"\",\n        minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        helpTip:\"\"\n      },\n\n      rightBtn: IconButton{\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        properties:{\n          toggle:true,\n        },\n        helpTip:\"\"\n      },\n    },\n\n    middleArrowBtnGrp: Group{\n      spacing: 0,\n      margins: [0,1,0,0],\n      orientation:\"row\",\n      alignment:[\"fill\",\"top\"],\n\n      leftBtn: IconButton{\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        properties:{\n          toggle:true,\n        },\n        helpTip:\"\"\n      },\n\n      a: StaticText{\n        text:\"\",\n        minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        helpTip:\"\"\n      },\n\n      centerBtn: IconButton{\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        properties:{\n          toggle:true,\n        },\n        helpTip:\"\"\n      },\n\n      a: StaticText{\n        text:\"\",\n        minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        helpTip:\"\"\n      },\n\n      rightBtn: IconButton{\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        properties:{\n          toggle:true,\n        },\n        helpTip:\"\"\n      },\n    },\n\n    bottomArrowBtnGrp: Group{\n      spacing: 0,\n      margins: [0,1,0,0],\n      orientation:\"row\",\n      alignment:[\"fill\",\"top\"],\n\n      leftBtn: IconButton{\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        properties:{\n          toggle:true,\n        },\n        helpTip:\"\"\n      },\n\n      a: StaticText{\n        text:\"\",\n        minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        helpTip:\"\"\n      },\n\n      centerBtn: IconButton{\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        properties:{\n          toggle:true,\n        },\n        helpTip:\"\"\n      },\n\n      a: StaticText{\n        text:\"\",\n        minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        helpTip:\"\"\n      },\n\n      rightBtn: IconButton{\n        minimumSize:[" + MAX_UI_HEIGHT_SIZE + ", " + MAX_ICON_HEIGHT_SIZE + "],\n        maximumSize:[5000, " + MAX_ICON_HEIGHT_SIZE + "],\n        alignment:[\"fill\",\"fill\"],\n        properties:{\n          toggle:true,\n        },\n        helpTip:\"\"\n      },\n    },\n  },\n\n  funGrp: Group{\n    spacing: 0,\n    margins:[0," + CC2014_UI_MARGINS + ",0,0],\n    alignment:[\"fill\",\"top\"],\n\n    applyBtn: Button{\n      text:\"Apply\",\n      minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[5000, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"fill\",\"fill\"],\n      helpTip:\"\"\n    },\n\n    a: StaticText{\n      text:\"\",\n      minimumSize:[" + CC2014_UI_MARGINS + ", " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[" + CC2014_UI_MARGINS + ", " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"fill\",\"fill\"],\n      helpTip:\"\"\n    },\n\n    helpBtn: Button{\n      text:\"?\",\n      minimumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n      maximumSize:[25, " + MAX_UI_HEIGHT_SIZE + "],\n      alignment:[\"right\",\"fill\"],\n      helpTip:\"\"\n    },\n  },\n}";
    thisUI.grp = thisUI.add(UIResource);
    thisUI.margins = [0, 0, 0, 0];
    thisUI.spacing = 0;
    thisUI.layout.layout(true);
    thisUI.layout.resize();
    thisUI.onResizing = thisUI.onResize = function() {
      return this.layout.resize();
    };
    try {
      thisUI.grp.arrowBtnGrp.topArrowBtnGrp.leftBtn.icon = ARROW_ICON[0];
      thisUI.grp.arrowBtnGrp.topArrowBtnGrp.centerBtn.icon = ARROW_ICON[1];
      thisUI.grp.arrowBtnGrp.topArrowBtnGrp.rightBtn.icon = ARROW_ICON[2];
      thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.leftBtn.icon = ARROW_ICON[3];
      thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.rightBtn.icon = ARROW_ICON[4];
      thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.leftBtn.icon = ARROW_ICON[5];
      thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.centerBtn.icon = ARROW_ICON[6];
      thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.rightBtn.icon = ARROW_ICON[7];
    } catch (error) {
      err = error;

      /*アイコンの生成に失敗した場合はテキストで表示する */
      thisUI.grp.arrowBtnGrp.topArrowBtnGrp.leftBtn.text = "→↓";
      thisUI.grp.arrowBtnGrp.topArrowBtnGrp.centerBtn.text = "↓";
      thisUI.grp.arrowBtnGrp.topArrowBtnGrp.rightBtn.text = "↓←";
      thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.leftBtn.text = "→";
      thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.centerBtn.text = "・";
      thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.rightBtn.text = "←";
      thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.leftBtn.text = "→↑";
      thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.centerBtn.text = "↑";
      thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.rightBtn.text = "↑←";
    }
    reDrawColorPanel = function() {
      thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorPanel.drawEllipse(RGBColor.RED);
      thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorPanel.drawEllipse(RGBColor.GREEN);
      thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorPanel.drawEllipse(RGBColor.BLUE);
      thisUI.grp.colorGrp.paletteGrp.colorGrp.setColorBtn.drawRect([thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorSlider.value / 255, thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorSlider.value / 255, thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorSlider.value / 255]);
    };
    thisUI.grp.addEventListener("mouseover", reDrawColorPanel);
    reDrawColorPanel();
    thisUI.grp.colorGrp.colorCodeGrp.subColorBtn1.color = colorStringToArray(SUB_PALETTE1_COLOR);
    thisUI.grp.colorGrp.colorCodeGrp.subColorBtn2.color = colorStringToArray(SUB_PALETTE2_COLOR);
    thisUI.grp.colorGrp.colorCodeGrp.subColorBtn3.color = colorStringToArray(SUB_PALETTE3_COLOR);
    thisUI.grp.colorGrp.paletteGrp.colorGrp.setColorBtn.drawRect(colorStringToArray(MAIN_PALETTE_COLOR));
    thisUI.grp.colorGrp.colorCodeGrp.subColorBtn1.drawRect(colorStringToArray(SUB_PALETTE1_COLOR));
    thisUI.grp.colorGrp.colorCodeGrp.subColorBtn2.drawRect(colorStringToArray(SUB_PALETTE2_COLOR));
    thisUI.grp.colorGrp.colorCodeGrp.subColorBtn3.drawRect(colorStringToArray(SUB_PALETTE3_COLOR));
    updateRGBColorText = function(color) {
      var colorCode;
      thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorEdit.text = Math.round(color[0] * COLOR_MAX_VALUE);
      thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorEdit.text = Math.round(color[1] * COLOR_MAX_VALUE);
      thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorEdit.text = Math.round(color[2] * COLOR_MAX_VALUE);
      colorCode = ("" + (Number(color[0] * COLOR_MAX_VALUE).toString(16)) + (Number(color[1] * COLOR_MAX_VALUE).toString(16)) + (Number(color[2] * COLOR_MAX_VALUE).toString(16))).toUpperCase();
      switch (colorCode) {
        case "FF00":
          colorCode = "FF0000";
          break;
        case "0FF0":
          colorCode = "00FF00";
          break;
        case "FF0FF":
          colorCode = "FF00FF";
          break;
        case "00FF":
          colorCode = "0000FF";
          break;
        case "000":
          colorCode = "000000";
      }
      thisUI.grp.colorGrp.colorCodeGrp.colorCodeText.text = colorCode;
      return 0;
    };
    updateRGBColorSlider = function(color) {
      thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorSlider.helpTip = thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorSlider.value = Math.round(Number(color[0]) * COLOR_MAX_VALUE);
      thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorSlider.helpTip = thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorSlider.value = Math.round(Number(color[1]) * COLOR_MAX_VALUE);
      thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorSlider.helpTip = thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorSlider.value = Math.round(Number(color[2]) * COLOR_MAX_VALUE);
      return 0;
    };
    updateRGBColorPanel = function(color) {
      thisUI.grp.colorGrp.paletteGrp.colorGrp.setColorBtn.drawRect(color);
      return 0;
    };
    updatePanelColor = function(color) {
      var actComp, rampEffect, selLayers;
      updateRGBColorText(color);
      updateRGBColorSlider(color);
      updateRGBColorPanel(color);
      app.settings.saveSetting(FPLData.getScriptURLName(), "MAIN_PALETTE_COLOR", color.parseColorNumber());
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        rampEffect.property(START_COLOR_MATCHNAME).setValueAtSmart(actComp.time, thisColor.getColorValue());
      }
      return 0;
    };
    thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorPanel.onClick = function() {
      thisColor.setColorValue(RGBColor.RED);
      updatePanelColor(thisColor.getColorValue());
      return 0;
    };
    thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorPanel.onClick = function() {
      thisColor.setColorValue(RGBColor.GREEN);
      updatePanelColor(thisColor.getColorValue());
      return 0;
    };
    thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorPanel.onClick = function() {
      thisColor.setColorValue(RGBColor.BLUE);
      updatePanelColor(thisColor.getColorValue());
      return 0;
    };
    thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorEdit.onChange = thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorEdit.onChange = thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorEdit.onChange = function() {
      if (!(this.text.length > 2)) {
        return;
      }
      if (isNaN(this.text)) {
        this.text = this.parent.colorSlider.minvalue;
      }
      if (this.text < this.parent.colorSlider.minvalue) {
        this.text = this.parent.colorSlider.minvalue;
      } else if (this.text > this.parent.colorSlider.maxvalue) {
        this.text = this.parent.colorSlider.maxvalue;
      }
      thisColor.setColorValue([thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorEdit.text.toString(10) / COLOR_MAX_VALUE, thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorEdit.text.toString(10) / COLOR_MAX_VALUE, thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorEdit.text.toString(10) / COLOR_MAX_VALUE]);
      updatePanelColor(thisColor.getColorValue());
      return 0;
    };
    thisUI.grp.colorGrp.paletteGrp.colorGrp.setColorBtn.onClick = function() {
      var actComp, colorArray, colorCode, rampEffect, selLayers;
      colorCode = $.colorPicker("0x" + (thisUI.grp.colorGrp.colorCodeGrp.colorCodeText.text.toUpperCase()));
      if (colorCode === ERR_PICKER) {
        return 0;
      }
      colorArray = colorStringToArray(colorCode);
      thisColor.setColorValue(colorArray);
      updatePanelColor(thisColor.getColorValue());
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        rampEffect.property(START_COLOR_MATCHNAME).setValueAtSmart(actComp.time, thisColor.getColorValue());
      }
      return 0;
    };
    thisUI.grp.colorGrp.colorCodeGrp.subColorBtn1.onClick = function() {
      var colorArray, colorCode;
      if (ScriptUI.environment.keyboardState.altKey) {
        updatePanelColor(this.color);
        return 0;
      }
      colorArray = this.color.parseColorNumber();
      colorCode = $.colorPicker(colorArray);
      if (colorCode === Number(ERR_PICKER)) {
        return 0;
      }
      this.color = colorStringToArray(colorCode);
      this.drawRect(this.color);
      app.settings.saveSetting(FPLData.getScriptURLName(), "SUB_PALETTE1_COLOR", this.color.parseColorNumber());
      return 0;
    };
    thisUI.grp.colorGrp.colorCodeGrp.subColorBtn2.onClick = function() {
      var colorArray, colorCode;
      if (ScriptUI.environment.keyboardState.altKey) {
        updatePanelColor(this.color);
        return 0;
      }
      colorArray = this.color.parseColorNumber();
      colorCode = $.colorPicker(colorArray);
      if (colorCode === Number(ERR_PICKER)) {
        return 0;
      }
      this.color = colorStringToArray(colorCode);
      this.drawRect(this.color);
      app.settings.saveSetting(FPLData.getScriptURLName(), "SUB_PALETTE2_COLOR", this.color.parseColorNumber());
      return 0;
    };
    thisUI.grp.colorGrp.colorCodeGrp.subColorBtn3.onClick = function() {
      var colorArray, colorCode;
      if (ScriptUI.environment.keyboardState.altKey) {
        updatePanelColor(this.color);
        return 0;
      }
      colorArray = this.color.parseColorNumber();
      colorCode = $.colorPicker(colorArray);
      if (colorCode === Number(ERR_PICKER)) {
        return 0;
      }
      this.color = colorStringToArray(colorCode);
      this.drawRect(this.color);
      app.settings.saveSetting(FPLData.getScriptURLName(), "SUB_PALETTE3_COLOR", this.color.parseColorNumber());
      return 0;
    };
    thisUI.grp.colorGrp.colorCodeGrp.colorCodeText.onChange = function() {
      if (this.text.length !== 6) {
        return;
      }
      thisColor.setColorValue([parseInt(this.text.substr(0, 2), 16) / COLOR_MAX_VALUE, parseInt(this.text.substr(2, 2), 16) / COLOR_MAX_VALUE, parseInt(this.text.substr(4, 2), 16) / COLOR_MAX_VALUE]);
      updatePanelColor(thisColor.getColorValue());
      return 0;
    };
    thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorSlider.onChanging = thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorSlider.onChanging = thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorSlider.onChanging = function() {
      thisColor.setColorValue([thisUI.grp.colorGrp.paletteGrp.controllGrp.rGrp.colorSlider.value / COLOR_MAX_VALUE, thisUI.grp.colorGrp.paletteGrp.controllGrp.gGrp.colorSlider.value / COLOR_MAX_VALUE, thisUI.grp.colorGrp.paletteGrp.controllGrp.bGrp.colorSlider.value / COLOR_MAX_VALUE]);
      updatePanelColor(thisColor.getColorValue());
      return 0;
    };
    thisUI.grp.opacityGrp.opacitySlider.onChanging = function() {
      thisUI.grp.opacityGrp.opacityEdit.text = Math.round(this.value);
      return 0;
    };
    thisUI.grp.opacityGrp.opacitySlider.onChange = function() {
      var actComp, rampEffect, selLayers;
      thisUI.grp.opacityGrp.opacityEdit.text = Math.round(this.value);
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        selLayers[selLayers.length - 1].property(ADBE_TRANSFORM_GROUP).property(ADBE_OPACITY).setValueAtSmart(actComp.time, this.value);
      }
      return 0;
    };
    thisUI.grp.opacityGrp.opacityEdit.onChange = function() {
      var actComp, rampEffect, selLayers;
      if (isNaN(this.text)) {
        this.text = this.parent.opacitySlider.maxvalue;
      }
      if (this.text < this.parent.opacitySlider.minvalue) {
        this.text = this.parent.opacitySlider.minvalue;
      } else if (this.text > this.parent.opacitySlider.maxvalue) {
        this.text = this.parent.opacitySlider.maxvalue;
      }
      this.parent.opacitySlider.value = this.text;
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        selLayers[selLayers.length - 1].property(ADBE_TRANSFORM_GROUP).property(ADBE_OPACITY).setValueAtSmart(actComp.time, this.text);
      }
      return 0;
    };
    thisUI.grp.blendModeGrp.paraBtn.onClick = function() {
      var actComp, markerValue, rampEffect, selLayers;
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        app.beginUndoGroup(FPLData.getScriptName());
        rampEffect.property(END_COLOR_MATCHNAME).setValueAtSmart(actComp.time, RGBColor.WHITE);
        selLayers[selLayers.length - 1].blendingMode = BlendingMode.MULTIPLY;
        selLayers[selLayers.length - 1].color = [0, 0, 0.8];
        selLayers[selLayers.length - 1].comment = getLocalizedText({
          jp: _STRINGS_JP.FLARE_C,
          en: _STRINGS_EN.FLARE_C
        });
        markerValue = new MarkerValue(getLocalizedText({
          jp: _STRINGS_JP.PARA_M,
          en: _STRINGS_EN.PARA_M
        }));
        selLayers[selLayers.length - 1].property(ADBE_MARKER).setValueAtTime(selLayers[selLayers.length - 1].inPoint, markerValue);
        app.endUndoGroup();
      }
      return 0;
    };
    thisUI.grp.blendModeGrp.flareBtn.onClick = function() {
      var actComp, markerValue, rampEffect, selLayers;
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        app.beginUndoGroup(FPLData.getScriptName());
        rampEffect.property(END_COLOR_MATCHNAME).setValueAtSmart(actComp.time, RGBColor.BLACK);
        selLayers[selLayers.length - 1].blendingMode = BlendingMode.SCREEN;
        selLayers[selLayers.length - 1].color = [1, 0.5, 0];
        selLayers[selLayers.length - 1].comment = getLocalizedText({
          jp: _STRINGS_JP.FLARE_C,
          en: _STRINGS_EN.FLARE_C
        });
        markerValue = new MarkerValue(getLocalizedText({
          jp: _STRINGS_JP.FLARE_M,
          en: _STRINGS_EN.FLARE_M
        }));
        selLayers[selLayers.length - 1].property(ADBE_MARKER).setValueAtTime(selLayers[selLayers.length - 1].inPoint, markerValue);
        app.endUndoGroup();
      }
      return 0;
    };
    thisUI.grp.curveTypeGrp.linearBtn.onClick = function() {
      var actComp, rampEffect, selLayers;
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        rampEffect.property(RAMP_SHAPE_MATCHNAME).setValueAtSmart(actComp.time, LINEAR_RAMP);
      }
      return 0;
    };
    thisUI.grp.curveTypeGrp.radiusBtn.onClick = function() {
      var actComp, rampEffect, selLayers;
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        rampEffect.property(RAMP_SHAPE_MATCHNAME).setValueAtSmart(actComp.time, RADIAL_RAMP);
      }
      return 0;
    };
    thisUI.grp.scatterGrp.scatterSlider.onChanging = function() {
      thisUI.grp.scatterGrp.scatterEdit.text = Math.round(this.value);
      return 0;
    };
    thisUI.grp.scatterGrp.scatterSlider.onChange = function() {
      var actComp, rampEffect, selLayers;
      thisUI.grp.scatterGrp.scatterEdit.text = Math.round(this.value);
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        rampEffect.property(RAMP_SCATTER_MATCHNAME).setValueAtSmart(actComp.time, this.value);
      }
      return 0;
    };
    thisUI.grp.scatterGrp.scatterEdit.onChange = function() {
      var actComp, rampEffect, selLayers;
      if (isNaN(this.text)) {
        this.text = this.parent.scatterSlider.minvalue;
      }
      if (this.text < this.parent.scatterSlider.minvalue) {
        this.text = this.parent.scatterSlider.minvalue;
      } else if (this.text > this.parent.scatterSlider.maxvalue) {
        this.text = this.parent.scatterSlider.maxvalue;
      }
      this.parent.scatterSlider.value = this.text;
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect != null) {
        rampEffect.property(RAMP_SCATTER_MATCHNAME).setValueAtSmart(actComp.time, this.text);
      }
      return 0;
    };
    thisUI.grp.arrowBtnGrp.topArrowBtnGrp.leftBtn.onClick = thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.leftBtn.onClick = thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.leftBtn.onClick = thisUI.grp.arrowBtnGrp.topArrowBtnGrp.centerBtn.onClick = thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.centerBtn.onClick = thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.centerBtn.onClick = thisUI.grp.arrowBtnGrp.topArrowBtnGrp.rightBtn.onClick = thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.rightBtn.onClick = thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.rightBtn.onClick = function() {
      var actComp, rampEffect, selLayers;
      thisUI.grp.arrowBtnGrp.topArrowBtnGrp.leftBtn.value = thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.leftBtn.value = thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.leftBtn.value = thisUI.grp.arrowBtnGrp.topArrowBtnGrp.centerBtn.value = thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.centerBtn.value = thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.centerBtn.value = thisUI.grp.arrowBtnGrp.topArrowBtnGrp.rightBtn.value = thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.rightBtn.value = thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.rightBtn.value = false;
      this.value = true;
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }
      selLayers = actComp.selectedLayers;
      if (!isLayerSelected(selLayers)) {
        return 0;
      }
      app.beginUndoGroup(FPLData.getScriptName());
      rampEffect = selLayers[selLayers.length - 1].property(ADBE_EFFECT_PARADE).property(ADBE_RAMP);
      if (rampEffect == null) {
        return 0;
      }
      if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.leftBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [0, 0]);
      } else if (thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.leftBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [0, actComp.height / 2]);
      } else if (thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.leftBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [0, actComp.height]);
      } else if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.centerBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, 0]);
      } else if (thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.centerBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, actComp.height / 2]);
      } else if (thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.centerBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, actComp.height]);
      } else if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.rightBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width, 0]);
      } else if (thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.rightBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width, actComp.height / 2]);
      } else if (thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.rightBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width, actComp.height]);
      }
      if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.centerBtn.value || thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.leftBtn.value || thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.centerBtn.value || thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.rightBtn.value || thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.centerBtn.value) {
        rampEffect.property(END_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, actComp.height / 2]);
      } else if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.leftBtn.value || thisUI.grp.arrowBtnGrp.topArrowBtnGrp.rightBtn.value) {

      } else {
        rampEffect.property(END_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, 0]);
      }
      if (thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.centerBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, actComp.height / 2]);
        rampEffect.property(END_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, 0]);
        rampEffect.property(RAMP_SHAPE_MATCHNAME).setValueAtSmart(actComp.time, RADIAL_RAMP);
        thisUI.grp.curveTypeGrp.radiusBtn.value = true;
      }
      app.endUndoGroup();
      return 0;
    };
    thisUI.grp.funGrp.helpBtn.onClick = function() {
      return alert((FPLData.getScriptName()) + " " + (FPLData.getScriptVersionNumber()) + "\n" + (getLocalizedText({
        jp: "" + _STRINGS_JP.AUTHOR,
        en: "" + _STRINGS_EN.AUTHOR
      })) + "\n\n" + (getLocalizedText({
        jp: "" + _STRINGS_JP.HELP,
        en: "" + _STRINGS_EN.HELP
      })), FPLData.getScriptName());
    };
    entryFunc = function() {
      var actComp, addedLayer, layerColor, markerValue, rampEffect;
      actComp = app.project.activeItem;
      if (!isCompActive(actComp)) {
        return 0;
      }

      /*レイヤーの色 */
      if (thisUI.grp.blendModeGrp.paraBtn.value) {
        layerColor = [0, 0, 0.8];
      } else {
        layerColor = [1, 0.5, 0];
      }
      addedLayer = actComp.layers.addSolid(layerColor, "Effect", actComp.width, actComp.height, actComp.pixelAspect, actComp.duration);
      rampEffect = addedLayer.property(ADBE_EFFECT_PARADE).addProperty(ADBE_RAMP);

      /*フレア/パラ コメント */
      if (thisUI.grp.blendModeGrp.paraBtn.value) {
        addedLayer.blendingMode = BlendingMode.MULTIPLY;
        addedLayer.comment = getLocalizedText({
          jp: _STRINGS_JP.FLARE_C,
          en: _STRINGS_EN.FLARE_C
        });
        markerValue = new MarkerValue(getLocalizedText({
          jp: _STRINGS_JP.PARA_M,
          en: _STRINGS_EN.PARA_M
        }));
        rampEffect.property(END_COLOR_MATCHNAME).setValueAtSmart(actComp.time, RGBColor.WHITE);
      } else {
        addedLayer.blendingMode = BlendingMode.SCREEN;
        addedLayer.comment = getLocalizedText({
          jp: _STRINGS_JP.FLARE_C,
          en: _STRINGS_EN.FLARE_C
        });
        markerValue = new MarkerValue(getLocalizedText({
          jp: _STRINGS_JP.FLARE_M,
          en: _STRINGS_EN.FLARE_M
        }));
        rampEffect.property(END_COLOR_MATCHNAME).setValueAtSmart(actComp.time, RGBColor.BLACK);
      }
      addedLayer.property(ADBE_MARKER).setValueAtTime(addedLayer.inPoint, markerValue);

      /*グラデーションのシェイプ */
      if (thisUI.grp.curveTypeGrp.linearBtn.value) {
        rampEffect.property(RAMP_SHAPE_MATCHNAME).setValueAtSmart(actComp.time, LINEAR_RAMP);
      } else {
        rampEffect.property(RAMP_SHAPE_MATCHNAME).setValueAtSmart(actComp.time, RADIAL_RAMP);
      }

      /*グラデーションの開始 */
      rampEffect.property(START_COLOR_MATCHNAME).setValueAtSmart(actComp.time, thisColor.getColorValue());

      /*グラデーションの終了 */
      if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.leftBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [0, 0]);
      } else if (thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.leftBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [0, actComp.height / 2]);
      } else if (thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.leftBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [0, actComp.height]);
      } else if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.centerBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, 0]);
      } else if (thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.centerBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, actComp.height / 2]);
        rampEffect.property(END_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, 0]);
        rampEffect.property(RAMP_SHAPE_MATCHNAME).setValueAtSmart(actComp.time, RADIAL_RAMP);
        thisUI.grp.curveTypeGrp.radiusBtn.value = true;
      } else if (thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.centerBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, actComp.height]);
      } else if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.rightBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width, 0]);
      } else if (thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.rightBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width, actComp.height / 2]);
      } else if (thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.rightBtn.value) {
        rampEffect.property(START_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width, actComp.height]);
      }
      if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.centerBtn.value || thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.leftBtn.value || thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.centerBtn.value || thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.rightBtn.value || thisUI.grp.arrowBtnGrp.bottomArrowBtnGrp.centerBtn.value) {
        rampEffect.property(END_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, actComp.height / 2]);
      } else if (thisUI.grp.arrowBtnGrp.topArrowBtnGrp.leftBtn.value || thisUI.grp.arrowBtnGrp.topArrowBtnGrp.rightBtn.value) {

      } else {
        rampEffect.property(END_OF_RAMP_MATCHNAME).setValueAtSmart(actComp.time, [actComp.width / 2, 0]);
      }
      return 0;
    };
    undoEntryFunc = function(data) {
      app.beginUndoGroup(data.getScriptName());
      entryFunc();
      app.endUndoGroup();
      return 0;
    };
    thisUI.grp.funGrp.applyBtn.onClick = function() {
      undoEntryFunc(FPLData);
      return 0;
    };
    thisUI.grp.arrowBtnGrp.middleArrowBtnGrp.centerBtn.value = true;
    if (!(thisUI instanceof Panel)) {
      thisUI.center();
      thisUI.show();
    }
    return thisUI;
  };


  /*start */

  if (!runAEVersionCheck(FPLData)) {
    return 0;
  }

  buildUI(this);

  return 0;

}).call(this);
